#include "ns3/applications-module.h"
#include "ns3/csma-module.h"
#include "ns3/core-module.h"
#include "ns3/internet-module.h"
#include "ns3/netanim-module.h"
#include "ns3/network-module.h"
#include "ns3/point-to-point-layout-module.h"
#include "ns3/point-to-point-module.h"
#include "ns3/mobility-helper.h"
using namespace ns3;

NS_LOG_COMPONENT_DEFINE("Star");

int
main(int argc, char* argv[])
{
    bool verbose = true;
    uint32_t nCsma = 3;
    uint32_t nSpokes = 4;

    CommandLine cmd(__FILE__);
    cmd.AddValue("nSpokes", "Number of nodes to place in the star", nSpokes);
    cmd.Parse(argc, argv);
    // For showing the status of transfers
    Time::SetResolution(Time::NS);
    LogComponentEnable("UdpEchoServerApplication", LOG_LEVEL_INFO);
    LogComponentEnable("UdpEchoClientApplication", LOG_LEVEL_INFO);

    NS_LOG_INFO("Build star topology.");
    // for point to point connections 
    PointToPointHelper pointToPoint;
    
    // Set attributes for point-to-point connections
    pointToPoint.SetDeviceAttribute("DataRate", StringValue("5Mbps"));
    pointToPoint.SetChannelAttribute("Delay", StringValue("2ms"));
    
    // Create a star topology (network 1)
    PointToPointStarHelper star(nSpokes, pointToPoint);
    
    // Intall internet stack on star network nodes
    NS_LOG_INFO("Install internet stack on all nodes.");
    InternetStackHelper internet;
    star.InstallStack(internet);
    
    // Assign IP Addresses to the nodes of star network
    NS_LOG_INFO("Assign IP Addresses.");
    star.AssignIpv4Addresses(Ipv4AddressHelper("10.1.1.0", "255.255.255.0"));

    //NS_LOG_INFO("Create applications.");
    //
    // Create a packet sink on the star "hub" to receive packets.
    //
    uint16_t port = 50000;
    Address hubLocalAddress(InetSocketAddress(Ipv4Address::GetAny(), port));
    PacketSinkHelper packetSinkHelper("ns3::TcpSocketFactory", hubLocalAddress);
    ApplicationContainer hubApp = packetSinkHelper.Install(star.GetHub());
    hubApp.Start(Seconds(1.0));
    hubApp.Stop(Seconds(10.0));
    
    // Create a CSMA network (network 2)
    nCsma = nCsma == 0 ? 1 : nCsma;
    
    NodeContainer p2pNodes;
    p2pNodes.Add(star.GetSpokeNode(3));
    p2pNodes.Create(1);
    
    NodeContainer csmaNodes;
    csmaNodes.Add(p2pNodes.Get(1));
    csmaNodes.Create(nCsma);
    
    NetDeviceContainer p2pDevices;
    p2pDevices = pointToPoint.Install(p2pNodes);

    CsmaHelper csma;
    csma.SetChannelAttribute("DataRate", StringValue("100Mbps"));
    csma.SetChannelAttribute("Delay", TimeValue(NanoSeconds(6560)));

    NetDeviceContainer csmaDevices;
    csmaDevices = csma.Install(csmaNodes);

    InternetStackHelper stack;
    //stack.Install(p2pNodes.Get(0));
    stack.Install(csmaNodes);
    
    Ipv4AddressHelper address;
    address.SetBase("10.3.0.0", "255.255.255.0");
    Ipv4InterfaceContainer p2pInterfaces;
    p2pInterfaces = address.Assign(p2pDevices);

    address.SetBase("10.2.0.0", "255.255.255.0");
    Ipv4InterfaceContainer csmaInterfaces;
    csmaInterfaces = address.Assign(csmaDevices);
 
     UdpEchoServerHelper echoServer(9);

    ApplicationContainer serverApps = echoServer.Install(csmaNodes.Get(2));
    serverApps.Start(Seconds(1.0));
    serverApps.Stop(Seconds(10.0));

    UdpEchoClientHelper echoClient(csmaInterfaces.GetAddress(2), 9);
    echoClient.SetAttribute("MaxPackets", UintegerValue(1));
    echoClient.SetAttribute("Interval", TimeValue(Seconds(1.0)));
    echoClient.SetAttribute("PacketSize", UintegerValue(1024));

    ApplicationContainer clientApps = echoClient.Install(star.GetSpokeNode(1));
    clientApps.Start(Seconds(2.0));
    clientApps.Stop(Seconds(10.0));
    
    
    NS_LOG_INFO("Enable static global routing.");
   
    Ipv4GlobalRoutingHelper::PopulateRoutingTables();

    NS_LOG_INFO("Enable pcap tracing.");
   
    pointToPoint.EnablePcapAll("star");
    star.BoundingBox(1, 1, 100, 100);
    MobilityHelper mobility;
    mobility.SetMobilityModel("ns3::ConstantPositionMobilityModel");
    mobility.Install(p2pNodes);
    mobility.Install(csmaNodes);
    AnimationInterface anim ("router.xml");
    anim.SetConstantPosition(p2pNodes.Get(1),400,200);
    anim.SetConstantPosition(csmaNodes.Get(1),600,150);
    anim.SetConstantPosition(csmaNodes.Get(2),700,150);
    anim.SetConstantPosition(csmaNodes.Get(3),700,150);


    NS_LOG_INFO("Run Simulation.");
    Simulator::Run();
    Simulator::Destroy();
    NS_LOG_INFO("Done.");

    return 0;
}


